﻿--------------------------------------------------------------------
/*
 SQL Server در Polybase
Site:        http://www.NikAmooz.com
Email:       Info@NikAmooz.com
Instagram:   https://instagram.com/nikamooz/
Telegram:	 https://telegram.me/nikamooz
Created By:  Masoud Taheri , Mehdi Ghapanvari 
*/
----------------------------------------------------------------------------------------------------------------------------------------
-- SourceDB ایجاد دیتابیس 
Use master
GO

If DB_ID (N'SourceDB') Is Not Null
Begin
Alter Database SourceDB Set Single_User With Rollback Immediate
Drop Database SourceDB
End
GO

Create Database SourceDB 
On Primary 
(Name = SourceDB, FileName = N'C:\Databases\Data\SourceDB.mdf', Size = 128 mb, FileGrowth = 32 mb, Maxsize = Unlimited),
FileGroup SourceDBFGData
(Name = SourceDBFData_1, FileName = N'C:\Databases\Data\SourceDBFData_1.ndf', Size = 256 mb, FileGrowth = 64 mb, Maxsize = Unlimited)
Log On
(Name = SourceDBLog, FileName = N'C:\Databases\Data\SourceDB.ldf', Size = 128 mb, FileGrowth = 32 mb, Maxsize = Unlimited)
GO 

Alter Database SourceDB Modify FileGroup SourceDBFGData Default
GO
-------------------------------------------------------------------------------------------------------------------------------------------
-- و اضافه نمودن داده به آن ها Employees و WorkTime ایجاد جداول
Use SourceDB
GO

Drop Table If Exists WorkTime
Drop Table If Exists Employees 
Create Table Employees 
(EmpID Int Not Null Primary Key,EmpFName Nvarchar(20) Not Null, EmpLName Nvarchar(30) Not Null, EmpAddress Nvarchar(50) Not Null, [Description] Nvarchar(100) Not Null)

; With Tbl (r) as (Select 1 Union All Select r + 1 From Tbl Where r < 50000)
Insert Into Employees
Select r, CONCAT('FName', r), CONCAT('LName', r), CONCAT('Address', r), CONCAT('Description', r) From Tbl
Option (Maxrecursion 0)
GO

Create Table WorkTime 
(EmpID Int Not Null,ClDate Date Not Null, EnterTime Time Not Null, ExitTime Time Not Null)

Create Unique Clustered Index IXCEmpIDClDate On WorkTime (EmpID, ClDate)
Alter Table WorkTime ADD Constraint FK_EmpID Foreign Key (EMPID) References Employees (EMPID)

;With TblTime (r) as (Select 1 Union All Select r + 1 From TblTime Where r < 29)
Insert Into WorkTime
Select EmpID, t.* From Employees Cross Apply(
Select Convert(Date,DATEADD(Day,r, '2020-01-31')) as ClDate, 
Convert(Time, DATEADD(MINUTE, ABS(CHECKSUM(NewId())) % 14, '07:30:00'), 108) EnterTime,
Convert(Time, DATEADD(MINUTE, ABS(CHECKSUM(NewId())) % 14, '16:45:00'), 108) ExitTime
From TblTime) as t
GO
-------------------------------------------------------------------------------------------------------------------------------------------------
-- ایجاد نام کاربری و رمز عبور
Use master
 GO

Create Login PolybaseLogin With Password = 'QwaL@9854E!!*B'

Use SourceDB
GO

Create User PolybaseUser For Login PolybaseLogin
GO
Grant Select On Employees To PolybaseUser
Grant Select On WorkTime To PolybaseUser
GO
-------------------------------------------------------------------------------------------------------------------------------------------------
-- SQLServerDB ایجاد دیتابیس 
Use master
GO

If DB_ID (N'SQLServerDB') Is Not Null
Begin
Alter Database SQLServerDB Set Single_User With Rollback Immediate
Drop Database SQLServerDB
End
GO

Create Database SQLServerDB 
On Primary 
(Name = SQLServerDB, FileName = N'C:\Databases\Data\SQLServerDB.mdf', Size = 128 mb, FileGrowth = 32 mb, Maxsize = Unlimited),
FileGroup SQLServerDBFGData
(Name = SQLServerDBFData_1, FileName = N'C:\Databases\Data\SQLServerDBFData_1.ndf', Size = 256 mb, FileGrowth = 64 mb, Maxsize = Unlimited)
Log On
(Name = SQLServerDBLog, FileName = N'C:\Databases\Data\SQLServerDB.ldf', Size = 128 mb, FileGrowth = 32 mb, Maxsize = Unlimited)
GO 

Alter Database SQLServerDB Modify FileGroup SQLServerDBFGData Default
GO
-------------------------------------------------------------------------------------------------------------------------------------------------
-- Database Scoped Credential و Master Key ایجاد
Use SQLServerDB
GO

CREATE MASTER KEY ENCRYPTION BY PASSWORD ='Pk$124*!88&' 
GO

CREATE DATABASE SCOPED CREDENTIAL SqlServerCredentials
WITH IDENTITY = 'PolybaseLogin', SECRET = 'QwaL@9854E!!*B';

-------------------------------------------------------------------------------------------------------------------------------------------------
-- External Data Source ایجاد
CREATE EXTERNAL DATA SOURCE SQLServerDataSource
WITH (
LOCATION = 'sqlserver://192.168.1.4:1433' ,
CREDENTIAL = SQLServerCredentials
);
GO
-------------------------------------------------------------------------------------------------------------------------------------------------
-- External Tables ایجاد
CREATE EXTERNAL TABLE ExternalEmployess
(
 [EmpID] INT NOT NULL, 
 [EmpFName] NVARCHAR(20) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL, 
 [EmpLName] NVARCHAR(30) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL, 
 [EmpAddress] NVARCHAR(50) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL, 
 [Description] NVARCHAR(100) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL
)
 WITH (
 LOCATION='SourceDB.dbo.Employees',
 DATA_SOURCE= SQLServerDataSource
);
GO

CREATE EXTERNAL TABLE ExternalWorkTime
(
 [EmpID] INT NOT NULL, 
 [ClDate] DATE NOT NULL, 
 [EnterTime] TIME(7) NOT NULL, 
 [ExitTime] TIME(7) NOT NULL
)
 WITH (
 LOCATION='SourceDB.dbo.WorkTime',
 DATA_SOURCE= SQLServerDataSource
);
GO
-------------------------------------------------------------------------------------------------------------------------------------------------
-- Statistics ایجاد
CREATE STATISTICS EmpIDStats ON ExternalEmployess (EmpID, EmpFName, EmpLName)
WITH FULLSCAN;

CREATE STATISTICS EmpIDCLDateStats ON ExternalWorkTime (EmpID, ExitTime)
WITH FULLSCAN;
-------------------------------------------------------------------------------------------------------------------------------------------------
-- کوئری ماکزیمم زمان خروج یک کارمند خاص در بازه زمانی مشخص شده 
Select e.EmpID, e.EmpFName, e.EmpLName, Max(w.ExitTime) as MaxExitTime From ExternalEmployess e
Inner Join ExternalWorkTime w
 On e.EmpID = w.EmpID
Where w.CLDate BetWeen '2020-02-20' And '2020-02-27' And e.EmpID = 1000
Group By e.EmpID, e.EmpFName, e.EmpLName
-------------------------------------------------------------------------------------------------------------------------------------------------
-- 