﻿--------------------------------------------------------------------
/*
 SQL Server در Polybase
Site:        http://www.NikAmooz.com
Email:       Info@NikAmooz.com
Instagram:   https://instagram.com/nikamooz/
Telegram:	 https://telegram.me/nikamooz
Created By:  Masoud Taheri , Mehdi Ghapanvari 
*/
--------------------------------------------------------------------
-- SQLServerDB ایجاد دیتابیس
Use master
GO

If DB_ID (N'SQLServerDB') Is Not Null
Begin
Alter Database SQLServerDB Set Single_User With Rollback Immediate
Drop Database SQLServerDB
End
GO

Create Database SQLServerDB 
On Primary 
(Name = SQLServerDB, FileName = N'C:\Databases\Data\SQLServerDB.mdf', Size = 128 mb, FileGrowth = 32 mb, Maxsize = Unlimited),
FileGroup SQLServerDBFGData
(Name = SQLServerDBFData_1, FileName = N'C:\Databases\Data\SQLServerDBFData_1.ndf', Size = 256 mb, FileGrowth = 64 mb, Maxsize = Unlimited)
Log On
(Name = SQLServerDBLog, FileName = N'C:\Databases\Data\SQLServerDB.ldf', Size = 128 mb, FileGrowth = 32 mb, Maxsize = Unlimited)
GO 

Alter Database SQLServerDB Modify FileGroup SQLServerDBFGData Default
GO
---------------------------------------------------------------------------------------------------------------------------------------------------
-- Database Scoped Credential و Master Key ایجاد
Use SQLServerDB
GO

CREATE MASTER KEY ENCRYPTION BY PASSWORD ='Pk$124*!88&' 
GO

CREATE DATABASE SCOPED CREDENTIAL SqlServerCredentials
WITH IDENTITY = 'NewLogin', SECRET = 'Qc@f123&A!';
---------------------------------------------------------------------------------------------------------------------------------------------------
-- External Data Source ایجاد
CREATE EXTERNAL DATA SOURCE SQLServerDataSource
WITH (
LOCATION = 'sqlserver://192.168.1.4:1433',
CREDENTIAL = SQLServerCredentials
);
GO
---------------------------------------------------------------------------------------------------------------------------------------------------
-- External Table ایجاد
USE SQLServerDB
GO

CREATE EXTERNAL TABLE ExternalSalesInfo
(
 [ProductKey] INT NOT NULL, 
 [OrderDateKey] INT NOT NULL, 
 [DueDateKey] INT NOT NULL, 
 [ShipDateKey] INT NOT NULL, 
 [ResellerKey] INT NOT NULL, 
 [EmployeeKey] INT NOT NULL, 
 [PromotionKey] INT NOT NULL, 
 [CurrencyKey] INT NOT NULL, 
 [SalesTerritoryKey] INT NOT NULL, 
 [SalesOrderNumber] NVARCHAR(20) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL, 
 [SalesOrderLineNumber] TINYINT NOT NULL, 
 [RevisionNumber] TINYINT, 
 [OrderQuantity] SMALLINT, 
 [UnitPrice] MONEY, 
 [ExtendedAmount] MONEY, 
 [UnitPriceDiscountPct] FLOAT(53), 
 [DiscountAmount] FLOAT(53), 
 [ProductStandardCost] MONEY, 
 [TotalProductCost] MONEY, 
 [SalesAmount] MONEY, 
 [TaxAmt] MONEY, 
 [Freight] MONEY, 
 [CarrierTrackingNumber] NVARCHAR(25) COLLATE SQL_Latin1_General_CP1_CI_AS, 
 [CustomerPONumber] NVARCHAR(25) COLLATE SQL_Latin1_General_CP1_CI_AS)
 WITH (
 LOCATION='DemoPageOrganization.dbo.SalesInfo',
 DATA_SOURCE= SQLServerDataSource
);
---------------------------------------------------------------------------------------------------------------------------------------------------
-- Linked Server ایجاد
USE [master]
GO
EXEC master.dbo.sp_addlinkedserver @server = N'SECONDARYSERVER', @srvproduct=N'SQL Server'

GO
EXEC master.dbo.sp_serveroption @server=N'SECONDARYSERVER', @optname=N'collation compatible', @optvalue=N'false'
GO
EXEC master.dbo.sp_serveroption @server=N'SECONDARYSERVER', @optname=N'data access', @optvalue=N'true'
GO
EXEC master.dbo.sp_serveroption @server=N'SECONDARYSERVER', @optname=N'dist', @optvalue=N'false'
GO
EXEC master.dbo.sp_serveroption @server=N'SECONDARYSERVER', @optname=N'pub', @optvalue=N'false'
GO
EXEC master.dbo.sp_serveroption @server=N'SECONDARYSERVER', @optname=N'rpc', @optvalue=N'true'
GO
EXEC master.dbo.sp_serveroption @server=N'SECONDARYSERVER', @optname=N'rpc out', @optvalue=N'true'
GO
EXEC master.dbo.sp_serveroption @server=N'SECONDARYSERVER', @optname=N'sub', @optvalue=N'false'
GO
EXEC master.dbo.sp_serveroption @server=N'SECONDARYSERVER', @optname=N'connect timeout', @optvalue=N'0'
GO
EXEC master.dbo.sp_serveroption @server=N'SECONDARYSERVER', @optname=N'collation name', @optvalue=null
GO
EXEC master.dbo.sp_serveroption @server=N'SECONDARYSERVER', @optname=N'lazy schema validation', @optvalue=N'false'
GO
EXEC master.dbo.sp_serveroption @server=N'SECONDARYSERVER', @optname=N'query timeout', @optvalue=N'0'
GO
EXEC master.dbo.sp_serveroption @server=N'SECONDARYSERVER', @optname=N'use remote collation', @optvalue=N'true'
GO
EXEC master.dbo.sp_serveroption @server=N'SECONDARYSERVER', @optname=N'remote proc transaction promotion', @optvalue=N'true'
GO
USE [master]
GO
EXEC master.dbo.sp_addlinkedsrvlogin @rmtsrvname = N'SECONDARYSERVER', @locallogin = NULL , @useself = N'False', @rmtuser = N'NewLogin', @rmtpassword = N'Qc@f123&A!'
GO
---------------------------------------------------------------------------------------------------------------------------------------------------
-- نمایش ده ردیف اول
USE SQLServerDB
GO

Set Statistics Time On

Select Top (10) * from ExternalSalesInfo

Select Top (10) * from [SECONDARYSERVER].[DemoPageOrganization].[dbo].[SalesInfo]
---------------------------------------------------------------------------------------------------------------------------------------------------
-- مجموع فروش در سه ماه اول سال 2004 
Select SUM(OrderQuantity * UnitPrice) As Total from ExternalSalesInfo
Where OrderDateKey >= 20040101 And OrderDateKey < 20040401

Select SUM(OrderQuantity * UnitPrice) As Total from [SECONDARYSERVER].[DemoPageOrganization].[dbo].[SalesInfo]
Where OrderDateKey >= 20040101 And OrderDateKey < 20040401
---------------------------------------------------------------------------------------------------------------------------------------------------
-- Remote ایجاد پیوند میان جداول محلی با جدول
Use SQLServerDB
GO

Drop Table If Exists #ProductKeys
Create Table #ProductKeys
(ProductKey Int Not Null)
GO

Insert Into #ProductKeys Values (360),(361), (362), (363), (364), (365), (366), (367), (368), (369), (370)

Select Count(*) as TotalCount From #ProductKeys p Inner Join ExternalSalesInfo exi
On p.ProductKey = exi.ProductKey

Select Count(*) as TotalCount From #ProductKeys p Inner Join 
[SECONDARYSERVER].[DemoPageOrganization].[dbo].[SalesInfo] exi
On p.ProductKey = exi.ProductKey
---------------------------------------------------------------------------------------------------------------------------------------------------
-- SalesInfo ایجاد ایندکس بر روی جدول
Use DemoPageOrganization
GO

Create Unique Clustered Index IXCSalesOrderNumber On SalesInfo
(SalesOrderNumber) With (Data_Compression = Page)
GO

Create NonClustered Index IXNCDueDateKeyProductKey On SalesInfo
(DueDateKey, ProductKey)
Include (SalesTerritoryKey, Employeekey)
With (Data_COmpression = Page)
---------------------------------------------------------------------------------------------------------------------------------------------------
-- Statistics ایجاد
Create Statistics FirstStats ON ExternalSalesInfo
(DueDateKey, ProductKey, SalesTerritoryKey, Employeekey)
With FullScan
---------------------------------------------------------------------------------------------------------------------------------------------------
-- Pushdown فعال و غیر فعال نمودن
CREATE EXTERNAL DATA SOURCE [DataSourceName]
WITH (
LOCATION = 'sqlserver://sqlserver',
PUSHDOWN = ON | OFF,
CREDENTIAL = Credential
);
GO
---------------------------------------------------------------------------------------------------------------------------------------------------
-- Polybase بررسی یک مثال در 
Use SQLServerDB
GO

Drop Table If Exists #ProductKeys
Create Table #ProductKeys
(ProductKey Int Not Null)
GO

Insert Into #ProductKeys Values (360),(361), (362), (363), (364), (365), (366), (367), (368), (369), (370)
GO

Select SalesTerritoryKey, Employeekey, Count(*) as TotalCount 
From ExternalSalesInfo esi
Inner Join #ProductKeys p On esi.ProductKey = p.ProductKey
Where esi.DueDateKey >= 20030101 And esi.DueDateKey < 20040101
Group By SalesTerritoryKey, Employeekey
Option (Force ExternalPushdown
---------------------------------------------------------------------------------------------------------------------------------------------------